#!/cygdrive/c/Python24/python.exe
"""FileManifest.py: This is a class for automatically creating and
   verifying file manifests for installed code."""

# Copyright (C) 2005 Shawn Sherwood

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
# MA  02110-1301, USA.

import yaml, os
import md5

class FileManifest:
    """File manifest creation and verification class"""
    def __init__( self, root_dir, sub_dirs, manifest_path, \
                  md5_extensions=None ):
        """md5_extensions is a list of file extensions that 
           are md5summed as part of the file manifest"""
        if md5_extensions == None:
            md5_extensions = []
        self.md5_extensions = md5_extensions
        self.root_dir = root_dir
        self.manifest_dictionary = {}
        self.sub_dirs = sub_dirs
        self.manifest_path = manifest_path

    def create_manifest( self ):
        """Loop through subdirectories, collecting manifest data"""
        for inode in os.listdir( self.root_dir ):
            if inode in self.sub_dirs:
                temp_dictionary = {}
                full_path = os.path.join( self.root_dir, inode )
                if os.path.isdir( full_path ):
                    self.manifest_dictionary[inode] = \
                      self.create_path_dictionary( full_path, temp_dictionary )

    def write_manifest_file(self):
        """Write out the yaml for the manifest to the expected path"""
        yaml_string = yaml.dump( self.manifest_dictionary )
        handle = open( self.manifest_path, "w" )
        handle.write( yaml_string )
        handle.close()
        
    def create_path_dictionary( self, path, work_dict ):
        """Loop through the directories gathering data into the manifest"""
        for inode in os.listdir( path ):
            full_path = os.path.join( path, inode )
            relative_path = self.get_relative_path(full_path)
            if os.path.isdir( full_path ):
                self.create_path_dictionary( full_path, work_dict )
            elif os.path.isfile( full_path ):
                if inode.split('.')[-1].lower() in self.md5_extensions:
                    handle = open( full_path, 'rb' )
                    data = handle.read()
                    work_dict[relative_path] = md5.new( data ).hexdigest()
                    handle.close()
                else:
                    work_dict[relative_path] = ''
        return work_dict

    def get_relative_path( self, full_path ):
        """Utility function that creates a relative path from a full path"""
        path_from_subdir = full_path.split( self.root_dir + os.sep )[-1] 
        return( os.sep.join( path_from_subdir.split( os.sep )[1:] ) )

    def load_manifest( self ):
        """Initialize manifest_dictionary from the existing manifest_path"""
        yaml_string = open( self.manifest_path, 'r' ).read()
        self.manifest_dictionary = yaml.load(yaml_string)

    def verify_manifest( self, mapping_dict ):
        """Loop through keys in manifest file, checking for files and
           md5sums as necessary. The mapping dictionary maps directories
           from the manifest file to directories in the file system."""
        tuple_check_list = []
        for subdir in self.manifest_dictionary.keys():
            for inode in self.manifest_dictionary[subdir]:
                new_tuple = ( os.path.join( mapping_dict[subdir], inode ),
                              self.manifest_dictionary[subdir][inode] )
                tuple_check_list.append( new_tuple )
        error_list = self.verify_file_md5_tuples( tuple_check_list )
        return( error_list )

    @classmethod
    def verify_file_md5_tuples(cls, file_md5_tuple_list):
        "Verify that a list of filepath, md5 pairs match"
        error_list = []
        for file_tuple in file_md5_tuple_list:
            filepath = file_tuple[0]
            md5sum = file_tuple[1]
            lastslash = filepath.rfind( os.sep )+1
            base = filepath[0:lastslash]
            if not os.path.isdir(base):
                err_string = "missing directory"
                error_list.append((filepath, err_string))
            elif not os.path.isfile(filepath):
                err_string = "missing file"
                error_list.append((filepath, err_string))
            elif md5sum != '':
                computed = md5.new(open(filepath, 'rb').read()).hexdigest()
                if md5sum != computed:
                    err_string = "invalid checksum: Actual: %s Expected %s" \
                                 % (computed, md5sum)
                    error_list.append((filepath, err_string))
        return error_list




#!/usr/bin/env python
"Provides a standard superclass for all Bombardier packages"

import re, os, yaml
from mini_utility import get_spkg_path
import sys, inspect
from static_data import OK, FAIL
from Logger import Logger
from Config import Config

def double_escape(old_string):
    'sometimes stuff needs this for certain reasons.'
    out_string = ''
    for i in old_string:
        if i == '\\':
            out_string += "\\\\"
        else:
            out_string += i
    return out_string

class SpkgException( Exception ):
    "raised when a system call is made that fails"
    def __init__(self, err_msg=""):
        self.err_msg = str(err_msg)
        err = Exception()
        Exception.__init__(err)
    def __str__(self):
        return self.err_msg
    def __repr__(self):
        return self.err_msg

def get_instance():
    "provides the name of the machine this is running on"
    path = os.getcwd()
    spkg_path = get_spkg_path()
    sub_dir = path.split(spkg_path)[1]
    instance_name = sub_dir.split(os.path.sep)[1]
    return instance_name

def get_config():
    "obtains the configuration object"
    instance_name = get_instance()
    config = Config(instance_name)
    return config

def main(cls):
    'A main method that can be used for troubleshooting'
    config = get_config()
    obj = cls(config)
    action = sys.argv[-1].lower()
    status = OK
    exec("status = obj.%s()" % action)
    sys.exit(status)

def mainV4(cls):
    'A main method that can be used for troubleshooting'
    config = get_config()
    obj = cls(config, Logger)
    action = sys.argv[-1].lower()
    status = OK
    if action == "install":
        status = obj.installer()
    elif action == "uninstall":
        status = obj.uninstaller()
    elif action == "configure":
        status = obj.configure()
    elif action == "verify":
        status = obj.verify()
    else:
        print "Unknown action %s" % action
        status = FAIL
    sys.exit(status)

def dumpReport(report, logger):
    "Legacy"
    instance_name = get_instance()
    output_path = os.path.join(get_spkg_path(), instance_name, "output")
    if not os.path.isdir(output_path):
        os.makedirs(output_path)
    script_name = sys.argv[-1].split(".py")[0]
    output_file = "%s-output.yml" % script_name
    yaml_string = yaml.dump(report)
    open(os.path.join(output_path, output_file), 'w').write(yaml_string)
    for line in yaml_string.split('\n'):
        logger.info("==REPORT==:%s" % line)

class SpkgV5:
    "a type-five abstract package"

    def __init__(self, config):
        '''
        config -- configuarion data object
        '''
        self.this_package_name = self._getname()
        self.instance_name     = config.get_instance()
        self.report            = {}
        self.logger            = Logger
        self.config            = config

    def _dump_report(self, report = None):
        '''
        command -- the command that was run
        report -- a dictionary of the data that was generated
        For a command that was run on an spkg class, this provides
        data back to the CNM 
        '''
        if type(report) != type({}):
            report = self.report
        command = inspect.stack()[2][3]
        output_path = os.path.join(get_spkg_path(), self.instance_name, "output")
        if not os.path.isdir(output_path):
            os.makedirs(output_path)
        output_file = "%s-output.yml" % command
        yaml_string = yaml.dump(report)
        open(os.path.join(output_path, output_file), 'w').write(yaml_string)
        for line in yaml_string.split('\n'):
            Logger.info("==REPORT==:%s" % line)

    def _check_status(self, status, err_msg="FAILED"):
        'small convenience function'
        if status != OK:
            raise SpkgException(err_msg)

    def _system(self, command, err_msg=""):
        '''run a command and raise if there's an error'''
        err_msg = command
        self._check_status( os.system( command ), err_msg )

    def _debug(self, string):
        'log a debug-level message'
        Logger.debug("[%s]|%s" % (self.this_package_name, string))

    def _info(self, string):
        'log a info-level message'
        Logger.info("[%s]|%s" % (self.this_package_name, string))

    def _warning(self, string):
        'log a warning-level message'
        Logger.warning("[%s]|%s" % (self.this_package_name, string))

    def _error(self, string):
        'log a error-level message'
        Logger.error("[%s]|%s" % (self.this_package_name, string))

    def _critical(self, string):
        'log a critical-level message'
        Logger.critical("[%s]|%s" % (self.this_package_name, string))

    def _getname(self):
        "Provides the name of this package based on directory structure"        
        cwd = os.getcwd()
        path = cwd.split(os.sep)
        return path[-1]

    def _abstract(self):
        "A generic abstract method"
        self._error("Attempting to call an abstract method")
        return FAIL

    def configure(self):
        """*configure* is the canonical method of configuring a package
        This should modify any settings that can be done on the fly"""
        return self._abstract()

    def verify(self):
        """*verify* is the canonical method of validating that a package
        is installed correctly. Doesn't necessary indicate that it is running
        or providing any services"""
        return self._abstract()

    def install(self):
        """*install* is the canonical method for installing a package the
        first time. Bombardier will call *verify* after install whenever
        a package is installed"""
        return self._abstract()

    def uninstall(self):
        """*uninstall* is the canonical method for removing a package.
        the package maintainer should always strive to remove all trace
        of a package so that the system is identical to the state it was
        in before the package was installed."""
        return self._abstract()

    def _modify_template_string(self, input_string, output_file, encoding=None,
                               process_escape = False):
        '''
        Goes through a string and replaces templates using member variables
        defined in this class
        input_string -- a string that may or may not have templated place-
                        holder information in it
        output_file -- where to write out our templated string output
        encoding -- what encoding to use when outputting our data
        process_escape -- sometimes we need to deal with '\' characters
        '''
        var_match = re.compile("\%\((.*?)\)s")
        variables = var_match.findall(input_string)
        status = OK
        output = []
        for line in input_string.split('\n'):
            variables = var_match.findall(line)
            config_dict = {}
            if len(variables) == 0:
                output.append(line)
                continue
            for variable in variables:
                if hasattr(self, variable):
                    config_value = getattr(self, variable)
                    if process_escape:
                        config_dict[variable] = double_escape(config_value)
                    else:
                        config_dict[variable] = config_value
                else:
                    msg = 'A variable was found in template "%s" for which '\
                          'there is no configuration value (variable: %s)'
                    self._error(msg  % (output_file, variable))
                    config_dict[variable] = 'UNKNOWN_TEMPLATE_VALUE'
                    status = FAIL
            if config_dict == {}:
                output.append(line)
            else:
                output.append(line % config_dict)
        output_data = '\n'.join(output)

        if encoding == None:
            open(output_file, 'w').write(output_data)
        else:
            open(output_file, 'wb').write(output_data.encode( encoding ))
        return status

    def _modify_template(self, input_file, output_file, encoding=None,
                        process_escape = False):
        'opens a file ane processes all template values in it'
        if encoding == None:
            input_string = open(input_file, 'r').read()
        else:
            input_string = unicode( open(input_file, 'rb').read(), encoding )
        self._info("Template: " + input_file )
        self._info("Created: " + output_file )
        return self._modify_template_string(input_string, output_file, encoding,
                                           process_escape)

class Spkg(SpkgV5):
    "Kept for backwards compatibility"
    def __init__(self, config, filesystem = None, logger = None):
        SpkgV5.__init__(self, config)
        self.thisPackagesName = self.this_package_name

    def checkStatus(self, status, err_msg="FAILED"):
        "Legacy"
        return self._check_status(status, err_msg)

    def installer(self):
        "Legacy"
        return self.install()

    def uninstaller(self):
        "Legacy"
        return self.uninstall()

    def modifyTemplateString(self, input_string, output_file, encoding=None,
                             process_escape = False):
        "Legacy"
        return self._modify_template_string(input_string, output_file, encoding,
                                           process_escape)

    def modifyTemplate(self, input_file, output_file, encoding=None,
                       process_escape = False):
        "Legacy"
        return self._modify_template(input_file, output_file, encoding,  
                                    process_escape)

    def dump_report(self, report = None):
        "Legacy"
        return self._dump_report(report)

    def system(self, command, err_msg=""):
        "Legacy"
        return self._system(command, err_msg)

    def debug(self, string):
        "Legacy"
        return self._debug(string)

    def info(self, string):
        "Legacy"
        return self._info(string)

    def warning(self, string):
        "Legacy"
        return self._warning(string)

    def error(self, string):
        "Legacy"
        return self._error(string)

    def critical(self, string):
        "Legacy"
        return self._critical(string)

class SpkgV4(Spkg):
    "Legacy"
    def __init__(self, config, filesystem = None, logger = None):
        Spkg.__init__(self, config, logger, filesystem)



#!/usr/bin/env python
"Used to encrypt configuration data"

import base64, binascii
from Cipher import AES
from static_data import CENSORED

VALID_CHARS = [ chr(x) for x in range(ord(' '), ord('~')+1) ] + ['\n']

class InvalidData(Exception):
    "Sometimes encrypted data can be corrupt or password is wrong"
    def __init__(self, dictionary, key, explanation = ''):
        """
        dictionary -- dictionary that we're trying to decrypt
        key -- pointer within dictionary
        explanation -- description of problem
        """
        Exception.__init__(self)
        self.key = key
        self.dictionary = dictionary
        if not explanation:
            explanation = "Invalid data"
        self.explanation = explanation
    def __str__(self):
        output = "%s: [key: (%s), value: (%s)]"
        output = output % (self.explanation, self.key, self.dictionary)
        return output
    def __repr__(self):
        return self.__str__()

class DecryptionException(Exception):
    "Lower-level error"
    def __init__(self, base64_text, reason):
        Exception.__init__(self)
        self.base64_text = base64_text
        self.reason = reason
    def __str__(self):
        return "Could not decrypt %s: %s" % (self.base64_text, self.reason)
    def __repr__(self):
        return self.__str__()

def pad(password):
    "Passwords have to be the appropriate length"
    output = password + '=' * (16 - len(password) % 16)
    return output

def get_invalid_characters(suspect_str, valid_chars):
    "determine which characters passed in are invalid"
    invalid_chars = []
    for char in suspect_str:
        if char not in valid_chars:
            invalid_chars.append( char )
    return invalid_chars

def _encrypt_string(plain_str, password):
    "encrypt a string with a password"
    encrypter  = AES.new(password, AES.MODE_ECB)
    base_64_str = pad(base64.encodestring(plain_str).strip())
    cipher_text = encrypter.encrypt(base_64_str)
    base64_cipher_str = base64.encodestring(cipher_text).strip()
    assert cipher_text == base64.decodestring(base64_cipher_str)
    return base64_cipher_str

def _decrypt_string(base64_cipher_str, password, valid_chars=VALID_CHARS):
    """Decrypt some base-64 encoded cypher-text with a password, checking 
    that the characters that come out are valid"""
    #print "base64_cipher_str: (%s)"%base64_cipher_str
    try:
        cipher_b64_str = base64.decodestring(base64_cipher_str)
        #print "cipher_b64_str: (%s)" %cipher_b64_str
        decrypter = AES.new(password, AES.MODE_ECB)
        base_64_str = decrypter.decrypt(cipher_b64_str)
        #print "base_64_str: ", base_64_str
        base = base_64_str.split('=')[0]
        base += "=" * (len(base) % 4)
        #print "base: ", base
        plain_str = base64.decodestring(base_64_str)
        #print "plain_str: ", plain_str
    except binascii.Error:
        msg = "Invalid characters in the decrypted text 0"
        raise DecryptionException(base64_cipher_str, msg)
    except ValueError:
        msg = "Invalid characters in the decrypted text 1"
        raise DecryptionException(base64_cipher_str, msg)
    invalid_chars = get_invalid_characters( plain_str, valid_chars )
    if invalid_chars:
        reason = "Invalid characters in the decrypted text: %s" % invalid_chars
        raise DecryptionException(base64_cipher_str, reason)
    return plain_str

class Cipher:
    "Class to provide simplified encryption services"

    def __init__(self, password):
        if password:
            password = pad(password)
        self.password = password

    def decrypt_string(self, string):
        return _decrypt_string(string, self.password)

    def encrypt_string(self, string):
        return _encrypt_string(string, self.password)

    def decrypt_dict(self, dictionary):
        "pad the password and decrypt the dictionary"
        self.decrypter_loop(dictionary)
        return dictionary

    def decrypter_loop(self, dictionary):
        """Recursive call to decrypt all values in a dictionary if password is
        given. If the password is not given, all encrypted values will be marked
        as censored."""
        if type(dictionary) != type({}):
            return
        for key in dictionary:
            value_type = type(dictionary[key])
            if value_type == type('') and type(key) == type('') \
              and key.startswith('enc_'):
                new_key = key.split("enc_")[-1]
                if self.password:
                    dictionary[new_key] = _decrypt_string(dictionary[key],
                                                         self.password)
                else:
                    dictionary[new_key] = CENSORED
                del dictionary[key]
            elif value_type == type({}):
                try:
                    self.decrypter_loop(dictionary[key])
                except DecryptionException:
                    raise InvalidData(dictionary[key], key, "Unable to decrypt")




"""Provides all of the infrastructure to perform two-way encryption,
compression, and splitting on files with full MD5 support to ensure
operational integrity."""

import md5, time, yaml, sys, os, glob

###### Constants #############################################################

ENCRYPT    = "encrypt"
COMPRESS   = "compress"
SPLIT      = "split"
JOIN       = "join"
DECRYPT    = "decrypt"
UNCOMPRESS = "uncompress"

MANIFEST_FILE = "manifest.yml"
OUTPUT_FREQUENCY = 25
MAX_FILE_SIZE = 500 * (1024 * 1024) # 50 MiB
BLOCK_SIZE = 124000

###### Exceptions ############################################################

class NoPrivateKeyException(Exception):
    "If encryption is required but no secret key has been provided"
    pass

class NoPossibleFilesException(Exception):
    """If we are trying to figure out how to 'do the right thing based on
    filename, and there are no files we can operate on"""
    pass

class ZeroOutputException(Exception):
    """We hate to write out zero-byte files. This is almost always due to a
    logic error. Therefore just throw an exception if it looks like we're
    about to do this"""
    pass

class Md5ValidationFailure(Exception):
    """What we throw when the file has gone through reverse processing and
    the expected MD5 hash does not match what we thought we'd see."""
    def __init__(self, file_name):
        Exception.__init__(self)
        self.msg = "Md5 failed validation in step: %s" % file_name
    def __str__(self):
        return self.msg
    def __repr__(self):
        return self.msg

class NoOptionsException(Exception):
    "If no options are set, there is nothing to do"
    pass

###### MD5 Validation ########################################################

def strip_directory(file_name):
    """Opposite of get_base_path. We're interested only in 
    the filename without path information"""
    base_path, sep, base_file_name = file_name.rpartition(os.path.sep)
    if not sep:
        return file_name
    return base_file_name

class Md5Validation:
    """A mix-in class for all file-processor objects to provide for md5 
    computation and checking. Some classes operate on more than one file
    and so our md5 operations are stored in a dictionary."""
    def __init__(self, file_name, target_md5_data, logger):
        self.file_name = file_name
        self.target_md5_data = target_md5_data
        self.logger = logger
        self.computed_md5_data = {}

    def update(self, data):
        "Gives us data that we will use to compute our md5 information."
        #print "UPDATE: %s || file_name: %s || len(data) %d" % (self, self.file_name, len(data))
        key = strip_directory(self.file_name)
        if not key in self.computed_md5_data:
            self.computed_md5_data[key] = md5.new()
        if not data:
            return
        self.computed_md5_data[key].update(data)

    def dump_md5(self):
        output = {}
        for key in self.computed_md5_data:
            output[key] = self.computed_md5_data[key].hexdigest()
        return output

    def check_md5(self):
        if len(self.computed_md5_data) == 0:
            #self.logger.warning("No MD5 data was computed")
            return
        for key in self.computed_md5_data:
            if not key in self.target_md5_data.keys():
                self.logger.error( "=DIFFERENT==============================" )
                self.logger.error("COMPUTED: %s" %  self.computed_md5_data.keys() )
                self.logger.error("TARGET: %s" % self.target_md5_data.keys())
                self.logger.error( "========================================" )
                raise Md5ValidationFailure(key)
            computed_hex_digest = self.computed_md5_data[key].hexdigest()
            if computed_hex_digest != self.target_md5_data[key]:
                self.logger.error( "=DIFFERENT MD5 (%10s)==============" % self )
                self.logger.error( "COMPUTED MD5: %s" % computed_hex_digest )
                self.logger.error( "TARGET MD5: %s" %self.target_md5_data[key] )
                self.logger.error( "========================================" )
                raise Md5ValidationFailure(key)

###### File readers and writers ##############################################

class AbstractFileWriter:
    """Writes a file to disk, doing nothing to it. Does not record MD5 
    data because it is no different than the previous processor"""
    def __init__(self, file_name, logger):
        self.logger = logger
        self.file_name = file_name
        self.file_handle = None

    def __repr__(self):
        return "ABSTRACT_FILE_WRITER"

    def open(self):
        if not self.file_handle:
            self.file_handle = open(self.file_name, 'wb')

    def set_max_file_size(self, max_file_size):
        "used by the testing harness only. Does nothing on normal file writers"
        pass

    def write(self, data):
        self.open()
        self.file_handle.write(data)

    def close(self):
        if not self.file_handle:
            raise ZeroOutputException()
        self.file_handle.flush()
        self.file_handle.close()
        self.file_handle = None

class AbstractFileReader:
    "Base class for reading files from the disk"
    def __init__(self, file_name, logger):
        self.logger     = logger
        self.block_size = BLOCK_SIZE

        self.original_size = os.stat(file_name)[6]
        self.size_so_far = 0
        self.output = 0
        self.file_name = file_name
        self.file_handle = None
        self.open()

    def __repr__(self):
        return "ABSTRACT_FILE_READER"

    def open(self):
        """This is a clutch function to be able to read the source file and 
        to wait until it's not busy any longer"""
        keep_trying = 0
        while True:
            try:
                self.file_handle = open(self.file_name, 'rb')
                break
            except IOError, ioe:
                msg = "Unable to read %s (%d)"
                self.logger.info(msg % (self.file_name, keep_trying))
                keep_trying += 1
                time.sleep(2)
                if keep_trying > 20:
                    raise ioe

    def read(self):
        self.output  += 1
        data = self.file_handle.read(self.block_size)
        self.size_so_far += len(data)
        if self.output > OUTPUT_FREQUENCY:
            done = (100.0*(float(self.size_so_far)/float(self.original_size)))
            self.logger.info("%3.0f percent done..." % done)
            self.output = 0
        return data

    def set_block_size(self, block_size):
        "used by the testing harness only"
        self.block_size = block_size

    def close(self):
        self.file_handle.close()

class FileReader(AbstractFileReader, Md5Validation):
    "This will read a file from the disk, doing nothing to it."
    def __init__(self, file_name, target_md5_data, logger):
        AbstractFileReader.__init__(self, file_name, logger)
        Md5Validation.__init__(self, file_name, target_md5_data, logger)

    def __repr__(self):
        return "FILE_READER"

    def read(self):
        data = AbstractFileReader.read(self)
        Md5Validation.update(self, data)
        return data

class JoinReader(AbstractFileReader, Md5Validation):
    """Reads a series of files from the disk, all ending in .part?, for 
    example, foo.part0, foo.part1, foo.part2... and join them back up"""
    def __init__(self, base_file_name, target_md5_data, logger):
        Md5Validation.__init__(self, base_file_name, target_md5_data, logger)
        self.file_number = 0
        self.base_file_name = base_file_name
        self.get_file_name()
        AbstractFileReader.__init__(self, self.file_name, logger)

    def __repr__(self):
        return "JOIN_READER"

    def get_file_name(self):
        self.file_name = "%s.part%d" % (self.base_file_name, self.file_number)

    def read(self):
        """Provides a read function similar to any file, but in this case
        will switch to a different file if the current file has run out."""
        data = AbstractFileReader.read(self)
        Md5Validation.update(self, data)
        while not data:
            self.file_number += 1
            self.get_file_name()
            if not os.path.isfile(self.file_name):
                return ''
            self.open()
            data = AbstractFileReader.read(self)
            Md5Validation.update(self, data)
        return data

class NullWriter(AbstractFileWriter, Md5Validation):
    """Does not write. This is used when the source and destination files are 
    the same thing, in which case we are merely verifying md5 hash values"""
    def __init__(self, logger):
        target_md5_data = {}
        AbstractFileWriter.__init__(self, '', None, logger)
        Md5Validation.__init__(self, '', target_md5_data, logger)
    def write(self, data):
        pass
    def open(self):
        pass
    def close(self):
        pass
    def __repr__(self):
        return "NULL_WRITER"

class FileWriter(AbstractFileWriter, Md5Validation):
    """Writes a file to disk, doing nothing to it. Does not record MD5 
    data because it is no different than the previous processor"""
    def __init__(self, base_file_name, target_md5_data, logger):
        target_md5_data = {}
        AbstractFileWriter.__init__(self, base_file_name, logger)
        Md5Validation.__init__(self, base_file_name, target_md5_data, logger)
    def __repr__(self):
        return "FILE_WRITER"


class SplitWriter(AbstractFileWriter, Md5Validation):
    "This class splits files into parts as it writes."
    def __init__(self, base_file_name, logger):
        target_md5_data = {}
        AbstractFileWriter.__init__(self, base_file_name, logger)
        Md5Validation.__init__(self, base_file_name, target_md5_data, logger)
        self.base_file_name = base_file_name
        self.max_file_size = MAX_FILE_SIZE
        self.file_size = 0
        self.file_number = 0

    def __repr__(self):
        return "SPLIT_WRITER"

    def open(self):
        if not self.file_handle:
            self.file_name = "%s.part%d" % (self.base_file_name, self.file_number)
            self.file_handle = open(self.file_name, 'wb')

    def set_max_file_size(self, max_file_size):
        "used by the testing harness only"
        self.max_file_size = max_file_size

    def write(self, data):
        self.open()
        self.file_size += len(data)

        if self.file_size > self.max_file_size:
            self.file_size = 0
            self.file_number += 1
            self.close()
            self.open()
        Md5Validation.update(self, data)
        self.file_handle.write(data)

###### Stream processors #####################################################

class AbstractProcessor:
    """Abstract class that can process a stream of data. These are intended
    to be chained together, the output from one procesor used as the input
    for the next, etc."""
    def __init__(self, logger):
        self.logger = logger
        self.block_size = BLOCK_SIZE
        self.read_buffer = ''
    def __repr__(self):
        return "PLUGGABLE_PROCESSOR_BASE_CLASS"
    def set_block_size(self, block_size):
        "used by the testing harness only"
        self.block_size = block_size  + (16 - block_size % 16)
    def write(self, data):
        """This allows us to process new data. What we return is the
        processed data for the next Pluggable Processor to act upon"""
        self.read_buffer = data
    def read(self):
        data = self.read_buffer
        self.read_buffer = ''
        return data
    def flush(self):
        """This gives us an opportunity to flush out any data we may 
        have saved with the idea that we can optimize our output later on"""
        return ''
    def suffix(self):
        "This influences the name of the written file"
        return ""

class Compressor(AbstractProcessor, Md5Validation):
    "provides bz2 compression"
    def __init__(self, base_file_name, logger):
        import bz2
        AbstractProcessor.__init__(self, logger)
        target_md5_data = {}
        file_name = base_file_name + self.suffix()
        Md5Validation.__init__(self, file_name, target_md5_data, logger)
        self.compressor = bz2.BZ2Compressor()
        self.read_buffer = ''
    def __repr__(self):
        return "COMPRESSOR"
    def write(self, data):
        data = self.compressor.compress(data)
        Md5Validation.update(self, data)
        self.read_buffer = data
    def flush(self):
        final_data = self.compressor.flush()
        Md5Validation.update(self, final_data)
        return final_data
    def suffix(self):
        return ".bz2"

class UnCompressor(AbstractProcessor, Md5Validation):
    "Reverse process of Compressor"
    def __init__(self, base_file_name, target_md5_data, logger):
        import bz2
        AbstractProcessor.__init__(self, logger)
        file_name = base_file_name + self.suffix()
        Md5Validation.__init__(self, file_name, target_md5_data, logger)
        self.decompressor = bz2.BZ2Decompressor()
    def __repr__(self):
        return "UNCOMPRESSOR"
    def write(self, data):
        Md5Validation.update(self, data)
        self.read_buffer = self.decompressor.decompress(data)
    def flush(self):
        return ''
    def suffix(self):
        return ".bz2"

class Encryptor(AbstractProcessor, Md5Validation):
    """Provides AES encryption. Everything the compressor class 
    does automatically, we have to do ourselves."""
    def __init__(self, base_file_name, private_key, logger):
        AbstractProcessor.__init__(self, logger)
        target_md5_data = {}
        file_name = base_file_name + self.suffix()
        Md5Validation.__init__(self, file_name, target_md5_data, logger)
        private_key += '=' * (16 - len(private_key) % 16)
        from Crypto.Cipher import AES
        self.encrypter = AES.new(private_key, AES.MODE_ECB)
        self.write_buffer = ''
    def __repr__(self):
        return "ENCRYPTOR"
    def write(self, data):
        "Can only process chunks of the right size"
        self.write_buffer += data
        if len(self.write_buffer) > self.block_size:
            self.read_buffer += self.encrypter.encrypt(self.write_buffer[:self.block_size])
            Md5Validation.update(self, self.read_buffer)
            self.write_buffer = self.write_buffer[self.block_size:]
    def flush(self):
        """AES requires blocks of 16 bytes, so we'll pad it
        and then mark the last byte with the length of 
        the pad for proper decryption"""
        pad_length = 15 - (len(self.write_buffer) % 16)
        pad = 'X'*pad_length
        pad += chr(pad_length)
        self.write_buffer += pad
        final_data = self.encrypter.encrypt(self.write_buffer)
        Md5Validation.update(self, final_data)
        return final_data
    def suffix(self):
        return ".enc"

class Decryptor(AbstractProcessor, Md5Validation):
    "Reverse of the Encryptor class"
    def __init__(self, base_file_name, private_key, target_md5_data, logger):
        AbstractProcessor.__init__(self, logger)
        file_name = base_file_name + self.suffix()
        Md5Validation.__init__(self, file_name, target_md5_data, logger)
        private_key += '=' * (16 - len(private_key) % 16)

        self.logger = logger
        from Crypto.Cipher import AES
        self.decryptor = AES.new(private_key, AES.MODE_ECB)
        self.write_buffer = ''
    def __repr__(self):
        return "DECRYPTOR"
    def write(self, data):
        """We validate the MD5 hash prior to decryption, because we 
        are comparing it to the output from the encryptor."""
        self.write_buffer += data
        if len(self.write_buffer) > self.block_size:
            self.read_buffer = self.decryptor.decrypt(self.write_buffer[:self.block_size])
            self.write_buffer = self.write_buffer[self.block_size:]
        Md5Validation.update(self, self.read_buffer)
    def flush(self):
        "Need to determine how much of the final data is pad and strip it"
        last_data_hunk = self.decryptor.decrypt(self.write_buffer)
        pad_size = ord(last_data_hunk[-1])
        if pad_size > 32:
            msg = "Invalid PAD size. Data is corrupt."
            self.logger.warning(msg)
            raise IOError
        good_data_length = len(last_data_hunk) - pad_size - 1
        
        final_data = last_data_hunk[:good_data_length]
        Md5Validation.update(self, final_data)
        return final_data

###### Pluggable processors ##################################################

def process_all(reader, processors, writer, priority):
    """Going through all the configured processors and writing 
    out a processed file"""
    new_data = reader.read()
    while new_data:
        time.sleep(0.01 * priority)
        for processor in processors:
            processor.write(new_data)
            new_data = processor.read()
        writer.write(new_data)
        new_data = reader.read()

    for index in range(0, len(processors)):
        processor = processors[index]
        final_data = processor.flush()
        if final_data:
            for processor in processors[index+1:]:
                processor.write(final_data)
                final_data = processor.read()
            writer.write(final_data)

    writer.close()
    reader.close()

def get_base_path(file_name):
    """return the directory that file_name is in. If it's not in a 
    separate directory, return nothing."""
    base_path, middle, end = file_name.rpartition(os.path.sep)
    if not middle:
        return ''
    return base_path

def get_manifest_data(file_name):
    manifest_path = os.path.join(get_base_path(file_name), MANIFEST_FILE)
    md5_dict = {}
    if os.path.isfile(manifest_path):
        md5_string = open(manifest_path, 'r').read()
        md5_dict = yaml.load(md5_string)
        if type(md5_dict) != type({}):
            md5_dict = {}
    return md5_dict

def get_suffix_file_name(file_name, processors):
    "Returns the name of the file we're currently working with"
    suffix_file_name = file_name
    for processor in processors:
        suffix_file_name += processor.suffix()
    return suffix_file_name

class ForwardPluggableFileProcessor:
    """This is for performing the 'forward' actions, i.e. encryption, 
    compression, and/or splitting. Takes options for what to do and
    writes out a manifest.yml file for md5 information."""

    def __init__(self, file_name, options,
                 logger, private_key=''):
        self.file_name         = file_name
        self.logger            = logger
        self.processors        = []
        if not options:
            raise NoOptionsException
            
        creation_file_name = self.file_name
        if COMPRESS in options:
            self.processors.append( Compressor(creation_file_name, self.logger) )
            creation_file_name += ".bz2"
        else:
            self.logger.warning("Not configured to compress.")

        if ENCRYPT in options:
            if not private_key:
                raise NoPrivateKeyException()
            try:
                encryptor = Encryptor(creation_file_name, private_key, self.logger)
                creation_file_name += ".enc"
                self.processors.append(encryptor)
            except ImportError:
                msg = "Encryption libraries are not installed. Not encrypting."
                self.logger.warning(msg)
        else:
            self.logger.warning("Not configured to encrypt.")

        target_md5_data = {}
        suffix_file_name = get_suffix_file_name(self.file_name, self.processors)
        self.reader = FileReader(self.file_name, target_md5_data, self.logger)
        if SPLIT in options:
            self.writer = SplitWriter(creation_file_name, self.logger)
        else:
            self.writer = FileWriter(creation_file_name, target_md5_data,
                                     self.logger)

    def set_block_size(self, block_size):
        "used by the testing harness only"
        self.reader.set_block_size(block_size)
        for processor in self.processors:
            processor.set_block_size(block_size)

    def set_max_file_size(self, max_file_size):
        "used by the testing harness only"
        self.writer.set_max_file_size(max_file_size)

    def dump_md5_sums(self):
        "Returns a dictionary of all md5sums collected"
        md5_dict = {}
        md5_dict.update(self.reader.dump_md5())
        md5_dict.update(self.writer.dump_md5())
        for processor in self.processors:
            md5_dict.update(processor.dump_md5())
        return md5_dict

    def create_manifest(self):
        old_md5_data = get_manifest_data(self.file_name)
        md5_data = self.dump_md5_sums()
        md5_data.update(old_md5_data)
        yaml_string = yaml.dump(md5_data)
        manifest_path = os.path.join(get_base_path(self.file_name), MANIFEST_FILE)
        open(manifest_path, 'w').write(yaml_string)

    def process_all(self, priority=0):
        process_all(self.reader, self.processors, self.writer, priority)
        self.create_manifest()

class ReversePluggableFileProcessor:
    """For performing 'reverse' operations to undo the operations in
    ForwardPluggableFileProcessor: decrypt, uncompress, and join. This
    will look at a directory and 'do the right thing' with the files that
    are there, based on extension, and will validate against md5 hash
    information in the manifest.yml file"""
    def __init__(self, file_name, logger, private_key = ''):
        self.file_name         = file_name
        self.logger            = logger
        self.processors        = []

        target_md5_data = get_manifest_data(self.file_name)

        options = self.determine_options()
        source_file_name = self.source_file_name(options)
        if not options:
            self.writer = NullWriter(self.logger)
        else:
            self.writer = FileWriter(self.file_name, target_md5_data,
                                     self.logger)

        if JOIN in options:
            self.reader = JoinReader(source_file_name, target_md5_data,
                                     self.logger)
        else:
            self.reader = FileReader(source_file_name, target_md5_data,
                                     self.logger)
        if DECRYPT in options:
            if not private_key:
                raise NoPrivateKeyException()
            source_file_name = source_file_name.partition('.enc')[0]
            decryptor = Decryptor(source_file_name, private_key, 
                                  target_md5_data, self.logger)
            self.processors.append(decryptor)
        if UNCOMPRESS in options:
            source_file_name = source_file_name.partition('.bz2')[0]
            uncompressor = UnCompressor(source_file_name, target_md5_data,
                                        self.logger)
            self.processors.append(uncompressor)

    def determine_options(self):
        """Based on files in a directory, determine what options we should 
        do and what file(s) we should operate on."""
        if os.path.isfile(self.file_name):
            return []
        possible_files = glob.glob(self.file_name+"*")
        suffix_set = set()
        for fname in possible_files:
            new_suffix_list = fname.replace(self.file_name, '').split('.')
            suffix_set = suffix_set.union(new_suffix_list)
        options = []

        if "part0" in suffix_set:
            options.append(JOIN)
        if "enc" in suffix_set:
            options.append(DECRYPT)
        if "bz2" in suffix_set:
            options.append(UNCOMPRESS)
        return options

    def source_file_name(self, options):
        "Identify the name of the file we're working on"
        source_file_name = self.file_name
        if UNCOMPRESS in options:
            source_file_name += ".bz2"
        if DECRYPT in options:
            source_file_name += ".enc"
        return source_file_name

    def process_all(self, priority = 0):
        process_all(self.reader, self.processors, self.writer, priority)
        self.check_md5_sums()
        
    def check_md5_sums(self):
        manifest_path = os.path.join(get_base_path(self.file_name), MANIFEST_FILE)
        if os.path.isfile(manifest_path):
            self.reader.check_md5()
            for processor in self.processors:
                processor.check_md5()
            self.writer.check_md5()
        else:
            msg = "%s doesn't exist. Not checking MD5" % manifest_path
            self.logger.warning(msg)



#!/usr/bin/env python
"""This is a simple class for wrapping functions in the
native python logging module."""

import logging, sys, os, shutil
import logging.handlers
from mini_utility import get_spkg_path
from static_data import OK, FAIL, LOG_MAX_SIZE, LOGS_TO_KEEP, LOG_FILE

FORMAT_STRING = '%(asctime)s|%(levelname)s|%(message)s|'
LOGGER_NAME = "bombardier"

class LoggerClass:

    """This class implements a simple interface that other logging
    type classes also implement. This is the most complete
    implementation. """

    def __init__(self):
        spkg_path = get_spkg_path()
        self.python_logger = None
        self.formatter = None
        self.std_err_handler = None
        self.file_handler = None
        self.log_path = None
        if spkg_path:
            self.log_path = os.path.join(get_spkg_path(), LOG_FILE)
            if self.check_log_size() == FAIL:
                self.cycle_log()
            self.make_logger()

    def check_log_size(self):
        """ We were dealing with Windows, and couldn't use Python's
        fancy rolling logger effectively. We therefore hd to see
        if the log is too big and deal with it periodically. -
         pbanka
        """
        if not os.path.isfile(self.log_path):
            return
        size = os.stat(self.log_path)[6]
        if size > LOG_MAX_SIZE:
            return FAIL
        return OK

    def cycle_log(self):
        "Manually cycle logs"
        oldest_log_file  = "%s.%s" % (self.log_path, LOGS_TO_KEEP)
        if os.path.isfile(oldest_log_file):
            try:
                os.unlink(oldest_log_file)
            except OSError:
                return
        for i in range(LOGS_TO_KEEP-1, 0, -1):
            source_path = "%s.%s" % (self.log_path, i)
            if os.path.isfile(source_path):
                dest_path = "%s.%s" % (self.log_path, (i+1))
                shutil.copyfile(source_path, dest_path)
                os.unlink(source_path)
        shutil.copyfile(self.log_path, "%s.1" % self.log_path)
        try:
            os.unlink(self.log_path)
        except OSError:
            return

    def make_logger(self):
        "use the logging subsystem to obtain a logger"
        self.python_logger = logging.getLogger(LOGGER_NAME)
        try:
            self.file_handler = logging.FileHandler(self.log_path)
        except IOError:
            try:
                self.file_handler = logging.FileHandler(self.log_path)
            except IOError:
                print "Unable to log to %s" % self.log_path
                self.file_handler = logging.StreamHandler(sys.stderr)
        self.formatter = logging.Formatter(FORMAT_STRING)
        self.file_handler.setFormatter(self.formatter)
        self.python_logger.addHandler(self.file_handler)
        self.python_logger.setLevel(logging.DEBUG)

    def info(self, msg):
        "wrap python logging facility"
        self.python_logger.info(msg)

    def debug(self, msg):
        "wrap python logging facility"
        self.python_logger.debug(msg)

    def warning(self, msg):
        "wrap python logging facility"
        self.python_logger.warning(msg)

    def error(self, msg):
        "wrap python logging facility"
        self.python_logger.error(msg)

    def critical(self, msg):
        "wrap python logging facility"
        self.python_logger.critical(msg)

    def add_std_err_logging(self):
        "Get stderr logging in addition to file logging"
        self.std_err_handler = logging.StreamHandler(sys.stderr)
        self.std_err_handler.setFormatter(self.formatter)
        self.python_logger.addHandler(self.std_err_handler)

    def rm_file_logging(self):
        "Sometimes you don't want to log to a file"
        if self.file_handler:
            msg = "Removing logging to %s" % self.log_path
            self.python_logger.info(msg)
            self.python_logger.removeHandler(self.file_handler)
        else:
            msg = "Being told to remove nonexistent file logging handler."
            self.python_logger.warning(msg)

    def rm_std_err_logging(self):
        "Sometimes you're tired of seeing stderr logging"
        if self.std_err_handler:
            self.python_logger.info("Removing logging to standard error")
            self.python_logger.removeHandler(self.std_err_handler)
        else:
            msg = "Being told to remove nonexistent stderr handler."
            self.python_logger.warning(msg)

Logger = LoggerClass()
